{% load i18n %}

<p>
{% blocktrans asvar text %}Hello,
your registration was canceled.{% endblocktrans %}{{ text|linebreaks }}
</p>

{% include 'leprikon/registration_mail_details.html' %}

{% include 'leprikon/registration_mail_footer.html' %}
