{% load i18n leprikon_tags %}

{% for reg in registrations %}
<a class="btn btn-default btn-xs" href="{% url 'leprikon:registration_list' %}#{{ reg.slug }}">{% trans 'registration' %}: {{ reg }}</a>
{% endfor %}

{% if subject.registration_allowed %}
    <a href="{{ registration_url }}" class="btn btn-primary btn-xs">
        {% trans 'register' %}
    </a>
{% else %}
    {% trans 'Registering is currently not allowed.' %}
    {% if subject.full %}{% trans 'The capacity is full.' %}{% endif %}
{% endif %}
