{% load i18n %}

<p>
{% blocktrans asvar text %}Hello,
we are sorry to inform You, that your registration was refused.{% endblocktrans %}
</p>

{% include 'leprikon/registration_mail_details.html' %}

{% include 'leprikon/registration_mail_footer.html' %}
