{% extends 'leprikon/default.html' %}
{% load i18n cms_tags leprikon_tags %}

{% block content %}

<h1>{% block title %}{% trans 'Summary' %}{% endblock %}</h1>

{% static_placeholder "summary" %}

<h2>{% trans 'New messages' %}</h2>

{% if new_messages.count %}
{% include 'leprikon/messages_table.html' with object_list=new_messages %}
{% else %}
<p><i>{% trans 'You have no new messages.' %}</i></p>
{% endif %}

<h2>{% trans 'Registrations and payments overview' %}</h2>
<p>
    <strong style="color: {{ payment_status.color }}">{{ payment_status.title }}</strong>
    ({% trans 'registrations count' %}: {{ registrations_count }},
    {% trans 'total price of all registrations' %}: {{ payment_status.price | currency }},
    {% trans 'discounts' %}: {{ payment_status.discount | currency }},
    {% trans 'paid' %}: {{ payment_status.paid | currency }})
</p>

{% endblock %}
