# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-10-04 20:36
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bankreader', '0001_initial'),
        ('leprikon', '0021_user_agreement'),
    ]

    operations = [
        migrations.AddField(
            model_name='subjectpayment',
            name='bankreader_transaction',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bankreader.Transaction', verbose_name='bank account transaction'),
        ),
    ]
